package a1MVC;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * The WorldRecord class implements a model of the data that describes a world
 * record, which consists of an event name, a time, a record holder (the person
 * who achieved that time), a nation (the country of the person who established
 * the record), a place (where the record was established), and a date.
 */
public class Datum {

	private List<String> fields = new Vector<String>();
	private static List<String> fieldNames = new Vector<String>();

	private static final String DELIMITER = ",";

	public Datum(List<String> fieldValues) {
		for (String val : fieldValues) {
			fields.add(val);
		}
	}

	public static boolean canBeParsed(String line, int reqdNumberOfFields) {
		if (line == null)
			return false;
		StringTokenizer st = new StringTokenizer(line, DELIMITER);
		int n = st.countTokens();
		if (n != reqdNumberOfFields) {
			System.out.println("line: " + line);
			System.out.println("number of tokens: " + n);
			System.out.println("Data format error!");
			return false;
		}
		return true;

	}

	public Vector toVector() {
		Vector v = new Vector();
		for (String s : fields) {
			v.add(s);
		}
		return v;
	}

	public static Vector getVectorOfFieldNames() {
		Vector v = new Vector();
		for (String s : fieldNames) {
			v.add(s);
		}
		return v;
	}

	public static List<Datum> parseFile(String fileName) throws IOException {
		boolean headerOption = true;
		List<Datum> records = new Vector<Datum>();
		BufferedReader br = new BufferedReader(new FileReader(fileName));

		String line;

		List<String> columnNames = new Vector<String>();
		// read the column names into a Vector
		if (headerOption) {
			line = br.readLine();
			StringTokenizer st = new StringTokenizer(line, DELIMITER);
			while (st.hasMoreTokens())
				columnNames.add(st.nextToken());
		}
		fieldNames = columnNames;

		while (true) {
			line = br.readLine();
			if (line == null)
				break;

			List<String> fieldValues = new Vector<String>();
			StringTokenizer st = new StringTokenizer(line, DELIMITER);
			while (st.hasMoreTokens())
				fieldValues.add(st.nextToken());
			records.add(new Datum(fieldValues));
		}
		return records;
	}

	public static boolean isWellFormedInputFile(String fileName,
			int reqdNumberOfFields) {
		boolean result = true;
		// prepare to read data file
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(fileName));
		} catch (FileNotFoundException e) {
			System.out.println("File not found: " + fileName);
			result = false;
		}
		String line = null;
		// verify that each row contains the same number of elements
		while (true) {
			try {
				line = br.readLine();
			} catch (IOException e) {
				System.out.println("Error reading data file");
				result = false;
			}
			if (!Datum.canBeParsed(line, reqdNumberOfFields)) {
				result = false;
				break;
			}
		}
		// close file
		try {
			br.close();
		} catch (IOException e) {
			result = false;
		}
		return result;
	}

}
