package a1MVC;

import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.*;
import java.io.*;
import java.util.*;

/**
 * The WorldRecordsDataModel implements a model of the data that describes the
 * world records, which consists of a number of events, each of which has an
 * event name, a time, a record holder (the person who achieved that time), a
 * nation (the country of the person who established the record), a place (where
 * the record was established), and a date.
 */
public class WorldRecordsDataModel implements ListSelectionModel, TableModel,
		ListModel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	// Recall that the DefaultTableModel stores the rows of the table as a
	// vector.
	// Each element of the vector is a vector, the elements of which are the
	// row's elements for
	// each of the defined columns.
	DefaultTableModel tableModel;

	ListSelectionModel listSelectionModel;

	DefaultListModel listModel;

	private static final int REQD_NUM_FIELDS = 6;
	private static final int COUNTRY_FIELD_NUM = 3;

	/**
	 * presently, this constructor creates an instance of a
	 * WorldRecordsDataModel that is populated with data contained in the passed
	 * file, of which the first row provide the header options, and the tokens
	 * are delimited by commas
	 * 
	 * @throws IOException
	 */
	public WorldRecordsDataModel(String fileName) {
		List<Datum> list = null;
		try {
			Datum.isWellFormedInputFile(fileName, REQD_NUM_FIELDS);
			list = Datum.parseFile(fileName);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}

		Vector v = new Vector();
		for (Datum wr : list) {
			v.add(wr.toVector());
		}
		tableModel = new DefaultTableModel(v, Datum.getVectorOfFieldNames());
		listSelectionModel = new DefaultListSelectionModel();
		listModel = new DefaultListModel();
		refreshListModel();
	}

	public List<String> getListOfCountries() {
		List<String> v = new Vector<String>();
		for (Object w : tableModel.getDataVector()) {
			v.add(((Vector<String>) w).elementAt(COUNTRY_FIELD_NUM) + "");
		}
		return v;
	}

	private void refreshListModel() {
		Vector<String> dataColumn = this.getTallyOfRecordsByCountry();
		listModel.removeAllElements();
		for (String s : dataColumn) {
			listModel.addElement(s);
		}
		for (ListDataListener l : listModel.getListDataListeners()) {
			// System.out.println("&&&&" + l);
			// l.contentsChanged(new ListDataEvent(this,
			// ListDataEvent.CONTENTS_CHANGED, 0, listModel.size()));
		}
	}

	public Vector<String> getTallyOfRecordsByCountry() {
		List<String> v = getListOfCountries();
		Vector<String> w = new Vector<String>();
		Collections.sort(v);

		int recordCtr = 0;
		String prevCountry = "";

		for (String currCountry : v) {
			currCountry = currCountry.trim();
			if (!currCountry.equals(prevCountry)) {
				if (!prevCountry.equals(""))
					w.add(prevCountry + "\t" + recordCtr);
				recordCtr = 1;
			} else
				recordCtr++;
			prevCountry = currCountry;
		}
		w.add(prevCountry + "\t" + recordCtr);
		return w;
	}

	public void addTableModelListener(TableModelListener l) {
		System.out.println("adding TableModelListener: " + l);
		tableModel.addTableModelListener(l);
	}

	/**
	 * The body of this method is empty in the parent class AbstractTableModel.
	 * We need to define it.
	 * 
	 */
	public void setValueAt(Object value, int row, int col) {
		System.out.println("setValueAt invoked");
		tableModel.setValueAt(value, row, col);
		tableModel.fireTableCellUpdated(row, col);
		/*
		 * the fireTableCellUpdated method dispatches a TableModelEvent and
		 * whichever registered listeners will process the event. One of them is
		 * JTable, which responds by re-rendering the table.
		 * 
		 */
		refreshListModel();
		System.out.println("fireTableCellUpdated");
	}

	public Object getValueAt(int row, int col) {
		return tableModel.getValueAt(row, col);
	}

	public int getColumnCount() {
		return tableModel.getColumnCount();
	}

	public int getRowCount() {
		return tableModel.getRowCount();
	}

	public boolean isCellEditable(int row, int col) {
		return tableModel.isCellEditable(row, col);
	}

	public String getColumnName(int idx) {
		return tableModel.getColumnName(idx);
	}

	// //////////
	public void addListSelectionListener(ListSelectionListener x) {
		listSelectionModel.addListSelectionListener(x);
	}

	public void addSelectionInterval(int index0, int index1) {
		listSelectionModel.addSelectionInterval(index0, index1);
	}

	public void clearSelection() {
		listSelectionModel.clearSelection();
	}

	public int getAnchorSelectionIndex() {
		return listSelectionModel.getAnchorSelectionIndex();
	}

	public int getLeadSelectionIndex() {
		return listSelectionModel.getLeadSelectionIndex();
	}

	public int getMaxSelectionIndex() {
		return listSelectionModel.getMaxSelectionIndex();
	}

	public int getMinSelectionIndex() {
		return listSelectionModel.getMinSelectionIndex();
	}

	public int getSelectionMode() {
		return listSelectionModel.getSelectionMode();
	}

	public boolean getValueIsAdjusting() {
		return listSelectionModel.getValueIsAdjusting();
	}

	public void insertIndexInterval(int index, int length, boolean before) {
		listSelectionModel.insertIndexInterval(index, length, before);
	}

	public boolean isSelectedIndex(int index) {
		return listSelectionModel.isSelectedIndex(index);
	}

	public boolean isSelectionEmpty() {
		return listSelectionModel.isSelectionEmpty();
	}

	public void removeIndexInterval(int index0, int index1) {
		listSelectionModel.removeIndexInterval(index0, index1);
	}

	public void removeListSelectionListener(ListSelectionListener x) {
		listSelectionModel.removeListSelectionListener(x);
	}

	public void removeSelectionInterval(int index0, int index1) {
		listSelectionModel.removeIndexInterval(index0, index1);
	}

	public void setAnchorSelectionIndex(int index) {
		listSelectionModel.setAnchorSelectionIndex(index);
	}

	public void setSelectionInterval(int index0, int index1) {
		listSelectionModel.setSelectionInterval(index0, index1);
	}

	public void setSelectionMode(int selectionMode) {
		listSelectionModel.setSelectionMode(selectionMode);
	}

	public void setValueIsAdjusting(boolean valueIsAdjusting) {
		listSelectionModel.setValueIsAdjusting(valueIsAdjusting);
	}

	/**
	 * Returns the most specific superclass for all the cell values in the
	 * column. This is used by the JTable to set up a default renderer and
	 * editor for the column.
	 */
	public Class<?> getColumnClass(int columnIndex) {
		Class<?> x = null;
		try {
			x = Class.forName("java.lang.String");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(0);
		}
		return x;
	}

	public void removeTableModelListener(TableModelListener l) {
		tableModel.removeTableModelListener(l);
	}

	public void setLeadSelectionIndex(int index) {
		listSelectionModel.setLeadSelectionIndex(index);
	}

	public void setListSelectionModel(ListSelectionModel model) {
		listSelectionModel = model;
	}

	// ///////////////
	public void addListDataListener(ListDataListener arg0) {
		// for (ListDataListener l : listModel.getListDataListeners()) {
		// System.out.println("&&&&" + l);
		// }
		// System.out.println("adding ListDataListener: " + arg0);
		listModel.addListDataListener(arg0);
	}

	public Object getElementAt(int arg0) {
		return listModel.getElementAt(arg0);
	}

	public int getSize() {
		return listModel.getSize();
	}

	public void removeListDataListener(ListDataListener arg0) {
		// System.out.println("removing ListDataListener: " + arg0);
		listModel.removeListDataListener(arg0);
	}

}